/**
 * DevExtreme (ui/scheduler/appointmentSettingsGenerator.js)
 * Version: 20.2.2 (build 20280-0711)
 * Build date: Tue Oct 06 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;
var _date = _interopRequireDefault(require("../../core/utils/date"));
var _type = require("../../core/utils/type");
var _extend = require("../../core/utils/extend");
var _recurrence = require("./recurrence");
var _utilsTimeZone = _interopRequireDefault(require("./utils.timeZone.js"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest()
}

function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) {
        return
    }
    if ("string" === typeof o) {
        return _arrayLikeToArray(o, minLen)
    }
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if ("Object" === n && o.constructor) {
        n = o.constructor.name
    }
    if ("Map" === n || "Set" === n) {
        return Array.from(o)
    }
    if ("Arguments" === n || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) {
        return _arrayLikeToArray(o, minLen)
    }
}

function _arrayLikeToArray(arr, len) {
    if (null == len || len > arr.length) {
        len = arr.length
    }
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i]
    }
    return arr2
}

function _iterableToArrayLimit(arr, i) {
    if ("undefined" === typeof Symbol || !(Symbol.iterator in Object(arr))) {
        return
    }
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = void 0;
    try {
        for (var _s, _i = arr[Symbol.iterator](); !(_n = (_s = _i.next()).done); _n = true) {
            _arr.push(_s.value);
            if (i && _arr.length === i) {
                break
            }
        }
    } catch (err) {
        _d = true;
        _e = err
    } finally {
        try {
            if (!_n && null != _i.return) {
                _i.return()
            }
        } finally {
            if (_d) {
                throw _e
            }
        }
    }
    return _arr
}

function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) {
        return arr
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var toMs = _date.default.dateToMilliseconds;
var AppointmentSettingsGenerator = function() {
    function AppointmentSettingsGenerator(scheduler) {
        _classCallCheck(this, AppointmentSettingsGenerator);
        this.scheduler = scheduler
    }
    _createClass(AppointmentSettingsGenerator, [{
        key: "create",
        value: function(rawAppointment) {
            var scheduler = this.scheduler;
            var workspace = scheduler.getWorkSpace();
            var appointment = scheduler.createAppointmentAdapter(rawAppointment);
            var itemResources = scheduler._resourcesManager.getResourcesFromItem(rawAppointment);
            var appointmentList = this._createRecurrenceAppointments(appointment, appointment.duration);
            if (0 === appointmentList.length) {
                if (workspace.isVirtualScrolling()) {
                    var groupIndices = workspace._isVerticalGroupedWorkSpace() ? workspace._getGroupIndexes(itemResources) : [0];
                    groupIndices.forEach(function(groupIndex) {
                        appointmentList.push({
                            startDate: appointment.startDate,
                            endDate: appointment.endDate,
                            groupIndex: groupIndex
                        })
                    })
                } else {
                    appointmentList.push({
                        startDate: appointment.startDate,
                        endDate: appointment.endDate
                    })
                }
            }
            this._updateGroupIndices(appointmentList, itemResources);
            if (this._canProcessNotNativeTimezoneDates(appointmentList, appointment)) {
                appointmentList = this._getProcessedNotNativeTimezoneDates(appointmentList, appointment)
            }
            var gridAppointmentList = this._createGridAppointmentList(appointmentList);
            gridAppointmentList = this._cropAppointmentsByStartDayHour(gridAppointmentList, rawAppointment);
            gridAppointmentList = this._getProcessedLongAppointmentsIfRequired(gridAppointmentList, appointment);
            var allDay = this.scheduler.appointmentTakesAllDay(rawAppointment) && this.scheduler._workSpace.supportAllDayRow();
            return this._createAppointmentInfos(gridAppointmentList, itemResources, allDay)
        }
    }, {
        key: "_canProcessNotNativeTimezoneDates",
        value: function(appointmentList, appointment) {
            var timeZoneName = this.scheduler.option("timeZone");
            var isEqualLocalTimeZone = _utilsTimeZone.default.isEqualLocalTimeZone,
                hasDSTInLocalTimeZone = _utilsTimeZone.default.hasDSTInLocalTimeZone;
            var isRecurrence = appointmentList.length > 1;
            var isTimeZoneSet = !(0, _type.isEmptyObject)(timeZoneName);
            var isAppointmentTimeZoneSet = !(0, _type.isEmptyObject)(appointment.startDateTimeZone);
            if (!isRecurrence) {
                return false
            }
            if (!isTimeZoneSet && hasDSTInLocalTimeZone()) {
                return false
            }
            return isTimeZoneSet && !isAppointmentTimeZoneSet && !isEqualLocalTimeZone(timeZoneName)
        }
    }, {
        key: "_getProcessedNotNativeDateIfCrossDST",
        value: function(date, dateRangeOffset) {
            var newDate = new Date(date);
            var newDateMinusOneHour = new Date(newDate);
            newDateMinusOneHour.setHours(newDateMinusOneHour.getHours() - 1);
            var newDateOffset = this.timeZoneCalculator.getOffsets(newDate).common;
            var newDateMinusOneHourOffset = this.timeZoneCalculator.getOffsets(newDateMinusOneHour).common;
            if (newDateOffset !== newDateMinusOneHourOffset) {
                return 0
            }
            return dateRangeOffset
        }
    }, {
        key: "_getProcessedNotNativeTimezoneDates",
        value: function(appointmentList, appointment) {
            var _this = this;
            var startDateRange = appointmentList[0].startDate;
            var endDateRange = appointmentList[appointmentList.length - 1].endDate;
            var startDateRangeOffset = this.timeZoneCalculator.getOffsets(startDateRange).common;
            var endDateRangeOffset = this.timeZoneCalculator.getOffsets(endDateRange).common;
            var isChangeOffsetInRange = startDateRangeOffset !== endDateRangeOffset;
            if (isChangeOffsetInRange) {
                return appointmentList.map(function(a) {
                    var diffStartDateOffset = _this.timeZoneCalculator.getOffsets(appointment.startDate).common - _this.timeZoneCalculator.getOffsets(a.startDate).common;
                    var diffEndDateOffset = _this.timeZoneCalculator.getOffsets(appointment.endDate).common - _this.timeZoneCalculator.getOffsets(a.endDate).common;
                    if (diffStartDateOffset < 0) {
                        diffStartDateOffset = _this._getProcessedNotNativeDateIfCrossDST(a.startDate, diffStartDateOffset);
                        diffEndDateOffset = _this._getProcessedNotNativeDateIfCrossDST(a.endDate, diffEndDateOffset)
                    }
                    var newStartDate = new Date(a.startDate.getTime() + diffStartDateOffset * toMs("hour"));
                    var newEndDate = new Date(a.endDate.getTime() + diffEndDateOffset * toMs("hour"));
                    var testNewStartDate = _this.timeZoneCalculator.createDate(newStartDate, {
                        path: "toGrid"
                    });
                    var testNewEndDate = _this.timeZoneCalculator.createDate(newEndDate, {
                        path: "toGrid"
                    });
                    if (appointment.duration > testNewEndDate.getTime() - testNewStartDate.getTime()) {
                        newEndDate = new Date(newStartDate.getTime() + appointment.duration)
                    }
                    return {
                        startDate: newStartDate,
                        endDate: newEndDate
                    }
                })
            }
            return appointmentList
        }
    }, {
        key: "_getProcessedLongAppointmentsIfRequired",
        value: function(gridAppointmentList, appointment) {
            var _this2 = this;
            var rawAppointment = appointment.source();
            var allDay = this.scheduler.appointmentTakesAllDay(rawAppointment);
            var dateRange = this.scheduler._workSpace.getDateRange();
            var renderingStrategy = this.scheduler.getLayoutManager().getRenderingStrategyInstance();
            if (renderingStrategy.needSeparateAppointment(allDay)) {
                var longStartDateParts = [];
                var resultDates = [];
                gridAppointmentList.forEach(function(gridAppointment) {
                    var maxDate = new Date(dateRange[1]);
                    var endDateOfPart = renderingStrategy.normalizeEndDateByViewEnd(rawAppointment, gridAppointment.endDate);
                    longStartDateParts = _date.default.getDatesOfInterval(gridAppointment.startDate, endDateOfPart, {
                        milliseconds: _this2.scheduler.getWorkSpace().getIntervalDuration(allDay)
                    });
                    var list = longStartDateParts.filter(function(startDatePart) {
                        return new Date(startDatePart) < maxDate
                    }).map(function(date) {
                        return {
                            startDate: date,
                            endDate: new Date(new Date(date).setMilliseconds(appointment.duration)),
                            source: gridAppointment.source
                        }
                    });
                    resultDates = resultDates.concat(list)
                });
                gridAppointmentList = resultDates
            }
            return gridAppointmentList
        }
    }, {
        key: "_createGridAppointmentList",
        value: function(appointmentList) {
            var _this3 = this;
            return appointmentList.map(function(source) {
                var startDate = _this3.timeZoneCalculator.createDate(source.startDate, {
                    path: "toGrid"
                });
                var endDate = _this3.timeZoneCalculator.createDate(source.endDate, {
                    path: "toGrid"
                });
                return {
                    startDate: startDate,
                    endDate: endDate,
                    source: source
                }
            })
        }
    }, {
        key: "_updateGroupIndices",
        value: function(appointments, itemResources) {
            var workspace = this.scheduler.getWorkSpace();
            if (workspace.isVirtualScrolling()) {
                var groupIndices = workspace._isVerticalGroupedWorkSpace() ? workspace._getGroupIndexes(itemResources) : [0];
                groupIndices.forEach(function(groupIndex) {
                    appointments.forEach(function(appointment) {
                        return appointment.groupIndex = groupIndex
                    })
                })
            }
        }
    }, {
        key: "_createExtremeRecurrenceDates",
        value: function(rawAppointment) {
            var dateRange = this.scheduler._workSpace.getDateRange();
            var startViewDate = this.scheduler.appointmentTakesAllDay(rawAppointment) ? _date.default.trimTime(new Date(dateRange[0])) : dateRange[0];
            var commonTimeZone = this.scheduler.option("timeZone");
            var minRecurrenceDate = commonTimeZone ? this.scheduler.timeZoneCalculator.createDate(startViewDate, {
                path: "fromGrid"
            }) : startViewDate;
            var maxRecurrenceDate = commonTimeZone ? this.scheduler.timeZoneCalculator.createDate(dateRange[1], {
                path: "fromGrid"
            }) : dateRange[1];
            return [minRecurrenceDate, maxRecurrenceDate]
        }
    }, {
        key: "_createRecurrenceOptions",
        value: function(appointment) {
            var _this$_createExtremeR = this._createExtremeRecurrenceDates(appointment.source()),
                _this$_createExtremeR2 = _slicedToArray(_this$_createExtremeR, 2),
                minRecurrenceDate = _this$_createExtremeR2[0],
                maxRecurrenceDate = _this$_createExtremeR2[1];
            return {
                rule: appointment.recurrenceRule,
                exception: appointment.recurrenceException,
                min: minRecurrenceDate,
                max: maxRecurrenceDate,
                firstDayOfWeek: this.scheduler.getFirstDayOfWeek(),
                start: appointment.startDate,
                end: appointment.endDate
            }
        }
    }, {
        key: "_createRecurrenceAppointments",
        value: function(appointment, duration) {
            var option = this._createRecurrenceOptions(appointment);
            var generatedStartDates = (0, _recurrence.getRecurrenceProcessor)().generateDates(option);
            return generatedStartDates.map(function(date) {
                var utcDate = _utilsTimeZone.default.createUTCDateWithLocalOffset(date);
                utcDate.setTime(utcDate.getTime() + duration);
                var endDate = _utilsTimeZone.default.createDateFromUTCWithLocalOffset(utcDate);
                return {
                    startDate: new Date(date),
                    endDate: endDate
                }
            })
        }
    }, {
        key: "_cropAppointmentsByStartDayHour",
        value: function(appointments, rawAppointment) {
            var _this4 = this;
            var workspace = this.scheduler.getWorkSpace();
            var startDayHour = this.scheduler._getCurrentViewOption("startDayHour");
            var firstViewDate = this.scheduler.getStartViewDate();
            return appointments.map(function(appointment) {
                var startDate = new Date(appointment.startDate);
                var resultDate = new Date(appointment.startDate);
                if (workspace.isVirtualScrolling()) {
                    var groupIndex = appointment.source.groupIndex;
                    var viewDataProvider = workspace.viewDataProvider;
                    firstViewDate = viewDataProvider.getGroupCellStartDate(groupIndex, startDate);
                    if (!firstViewDate) {
                        firstViewDate = viewDataProvider.getGroupStartDate(groupIndex)
                    }
                    startDayHour = firstViewDate.getHours()
                }
                if (_this4.scheduler.appointmentTakesAllDay(rawAppointment)) {
                    resultDate = _date.default.normalizeDate(startDate, firstViewDate)
                } else {
                    if (startDate < firstViewDate) {
                        startDate = firstViewDate
                    }
                    resultDate = _date.default.normalizeDate(appointment.startDate, startDate)
                }
                appointment.startDate = _date.default.roundDateByStartDayHour(resultDate, startDayHour);
                return appointment
            })
        }
    }, {
        key: "_createAppointmentInfos",
        value: function(gridAppointments, appointmentResources, allDay) {
            var _this5 = this;
            var result = [];
            var _loop = function(i) {
                var coordinates = _this5.scheduler._workSpace.getCoordinatesByDateInGroup(gridAppointments[i].startDate, appointmentResources, allDay);
                coordinates.forEach(function(coordinate) {
                    (0, _extend.extend)(coordinate, {
                        info: {
                            appointment: gridAppointments[i],
                            sourceAppointment: gridAppointments[i].source
                        }
                    })
                });
                result = result.concat(coordinates)
            };
            for (var i = 0; i < gridAppointments.length; i++) {
                _loop(i)
            }
            return result
        }
    }, {
        key: "timeZoneCalculator",
        get: function() {
            return this.scheduler.timeZoneCalculator
        }
    }]);
    return AppointmentSettingsGenerator
}();
exports.default = AppointmentSettingsGenerator;
module.exports = exports.default;
